/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
package org.jdtaus.core.logging.ri.jdk14;

import java.util.logging.Level;
import org.jdtaus.core.container.ContainerInitializer;
import org.jdtaus.core.container.Dependency;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.ModelFactory;
import org.jdtaus.core.container.Properties;
import org.jdtaus.core.container.Property;
import org.jdtaus.core.container.PropertyException;
import org.jdtaus.core.logging.spi.Logger;

/**
 * jDTAUS Core SPI JDK 1.4 {@code Logger} implementation.
 * <p>The name of the JDK logger is specified by property {@code name}.
 * Property {@code name} defaults to {@code org.jdtaus.runtime}.</p>
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: JDK14Logger.java 4353 2007-08-23 02:14:13Z schulte2005 $
 *
 * @see org.jdtaus.core.container.Container
 * @see #initialize()
 */
public final class JDK14Logger implements ContainerInitializer, Logger
{
    //--Implementation----------------------------------------------------------

// <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:jdtausImplementation
    // This section is managed by jdtaus-container-mojo.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(JDK14Logger.class.getName());
// </editor-fold>//GEN-END:jdtausImplementation

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

// <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:jdtausConstructors
    // This section is managed by jdtaus-container-mojo.

    /**
     * <code>JDK14Logger</code> implementation constructor.
     *
     * @param meta Implementation meta-data.
     *
     * @throws NullPointerException if <code>meta</code> is <code>null</code>.
     */
    private JDK14Logger(final Implementation meta)
    {
        super();
        if(meta == null)
        {
            throw new NullPointerException("meta");
        }
        this.initializeProperties(meta.getProperties());
    }
    /**
     * <code>JDK14Logger</code> dependency constructor.
     *
     * @param meta dependency meta-data.
     *
     * @throws NullPointerException if <code>meta</code> is <code>null</code>.
     */
    private JDK14Logger(final Dependency meta)
    {
        super();
        if(meta == null)
        {
            throw new NullPointerException("meta");
        }
        this.initializeProperties(meta.getProperties());
    }

    /**
     * Initializes the properties of the instance.
     *
     * @param meta the property values to initialize the instance with.
     *
     * @throws NullPointerException if {@code meta} is {@code null}.
     */
    private void initializeProperties(final Properties meta)
    {
        Property p;

        if(meta == null)
        {
            throw new NullPointerException("meta");
        }

        p = meta.getProperty("name");
        this._name = (java.lang.String) p.getValue();

    }
// </editor-fold>//GEN-END:jdtausConstructors

    //------------------------------------------------------------Constructors--
    //--ContainerInitializer----------------------------------------------------

    /** JDK logger. */
    private java.util.logging.Logger logger;

    /**
     * Requests a JDK logger for the name given by property {@code name}.
     *
     * @see #assertValidProperties()
     */
    public void initialize()
    {
        this.assertValidProperties();
        this.logger = java.util.logging.Logger.getLogger(this.getName());
    }

    //----------------------------------------------------ContainerInitializer--
    //--Properties--------------------------------------------------------------

// <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:jdtausProperties
    // This section is managed by jdtaus-container-mojo.

    /**
     * Property {@code name}.
     * @serial
     */
    private java.lang.String _name;

    /**
     * Gets the value of property <code>name</code>.
     *
     * @return the value of property <code>name</code>.
     */
    public java.lang.String getName()
    {
        return this._name;
    }

// </editor-fold>//GEN-END:jdtausProperties

    //--------------------------------------------------------------Properties--
    //--Logger------------------------------------------------------------------

    public boolean isDebugEnabled()
    {
        return this.logger.isLoggable(Level.FINE);
    }

    public void debug(String message)
    {
        this.log(Level.FINE, message, null);
    }

    public void debug(Throwable throwable)
    {
        this.log(Level.FINE, throwable.getMessage(), throwable);
    }

    public boolean isErrorEnabled()
    {
        return this.logger.isLoggable(Level.SEVERE);
    }

    public void error(String message)
    {
        this.log(Level.SEVERE, message, null);
    }

    public void error(Throwable throwable)
    {
        this.log(Level.SEVERE, throwable.getMessage(), throwable);
    }

    public boolean isFatalEnabled()
    {
        return this.logger.isLoggable(Level.SEVERE);
    }

    public void fatal(String message)
    {
        this.log(Level.SEVERE, message, null);
    }

    public void fatal(Throwable throwable)
    {
        this.log(Level.SEVERE, throwable.getMessage(), throwable);
    }

    public boolean isInfoEnabled()
    {
        return this.logger.isLoggable(Level.INFO);
    }

    public void info(String message)
    {
        this.log(Level.INFO, message, null);
    }

    public void info(Throwable throwable)
    {
        this.log(Level.INFO, throwable.getMessage(), throwable);
    }

    public boolean isTraceEnabled()
    {
        return this.logger.isLoggable(Level.FINEST);
    }

    public void trace(String message)
    {
        this.log(Level.FINEST, message, null);
    }

    public void trace(Throwable throwable)
    {
        this.log(Level.FINEST, throwable.getMessage(), throwable);
    }

    public boolean isWarnEnabled()
    {
        return this.logger.isLoggable(Level.WARNING);
    }

    public void warn(String message)
    {
        this.log(Level.WARNING, message, null);
    }

    public void warn(Throwable throwable)
    {
        this.log(Level.WARNING, throwable.getMessage(), throwable);
    }

    //------------------------------------------------------------------Logger--
    //--JDK14Logger-------------------------------------------------------------

    /** Creates a new {@code JDK14Logger} instance. */
    public JDK14Logger()
    {
        this(META);
        this.initialize();
    }

    /**
     * Creates a new {@code JDK14Logger} instance taking the name to use for the
     * JDK logger.
     *
     * @param name the name to use for the JDK logger.
     *
     * @throws PropertyException if {@code name} is {@code null} or the length
     * of {@code name} is {@code 0}.
     */
    public JDK14Logger(final String name)
    {
        this(META);
        this._name = name;
        this.initialize();
    }

    /**
     * Checks configured properties.
     *
     * @throws PropertyError if property {@code name} is {@code null} or
     * of no length.
     */
    private void assertValidProperties()
    {
        if(this.getName() == null || this.getName().length() <= 0)
        {
            throw new PropertyException("name", this.getName());
        }
    }

    private void log(final Level level, final String msg, final Throwable t)
    {
        if (this.logger.isLoggable(level))
        {
            StackTraceElement caller;
            final Throwable x = new Throwable();
            final StackTraceElement[] elements = x.getStackTrace();

            String cname = "unknown";
            String method = "unknown";

            if(elements != null && elements.length > 2)
            {
                caller = elements[2];
                cname = caller.getClassName();
                method = caller.getMethodName();
            }

            if(t == null)
            {
                this.logger.logp(level, cname, method, msg);
            }
            else
            {
                this.logger.logp(level, cname, method, msg, t);
            }
        }
    }

    //-------------------------------------------------------------JDK14Logger--
}
